  /*------------------------------------------------------------------------------------------------------------
 IR-Fernbedienung Sony RM-AMU096: 12 wichtigste Tasten
  Realisiert mit ATtiny44
 
 Version 2.0  vom 5.12.2013
 Version 1.0  vom 9.8.2012
 
 
 Interner RC Oszillator mit 8MHz (Vorteiler /8 aus)
 
 
 
 Layout:

       LED+         VCC  GND
  BC817..1K..PCINT8 PB0  PA0 PCINT0 T_prev  \__Pause
    T_Power  PCINT9 PB1  PA1 PCINT1 T_next  /
              Reset PB3  PA2 PCINT2 T_CD   \___Sleep
  T_VolDown PCINT10 PB2  PA3 PCINT3 T_USB  /
    T_VolUp  PCINT7 PA7  PA4 PCINT4 T_WDR5
     T_WDR1  PCINT6 PA6  PA5 PCINT5 T_WDR2
 
  Die IR-LED hngt ber einen NPN-Transistor an PB0 und geht ber 20R an +3V.
  Der Transistor hat 1K Basisvorwiderstand und die Basis wird mit 47K gegen GND (Emitter) gezogen.
  Die Tasten verbinden GND mit einem Eingang. 
  Tasten die an 2 Eingngen hngen ziehen diese ber je zwei Dioden 1N4148 nach GND


  Tastenlayout:

  Power 
  WDR1 WDR2 WDR5
  Prev pause next
  CD USB sleep
  
  Vol-  Vol+



  Fuses: E2 DE FF  (Low High Extended)
*/

#define F_CPU 8000000UL	/* CPU clock in Hertz */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define U8 unsigned char
#define U16 unsigned int
#define U32 unsigned long
#define I8 signed char
#define I16 int

#define BIT0 1
#define BIT1 2
#define BIT2 4
#define BIT3 8
#define BIT4 16
#define BIT5 32
#define BIT6 64
#define BIT7 128
#define BIT8 256
#define BIT9 512




//--- IR-Sendecodes ---
//Achtung: LSB wird zuerst gesendet.
//Die Bitmuster auf dem Zettel sind also von rechts nach links einzutragen!

//Erfasst mit IR-LED und 47K Lastwiderstand

//                        12 Bit
//                      123456789 12
#define IR_POWER      0b100000010101,12
#define IR_TUNER      0b100000100001,12
#define IR_AUDIO_IN   0b100000011101,12
#define IR_CD         0b100000100101,12
#define IR_VOL_UP     0b100000010010,12
#define IR_VOL_DN     0b100000010011,12
#define IR_SLEEP      0b100001100000,12

//                        15 Bit
//                      123456789 12345
#define IR_USB        0b100100000011010,15
     
//                        20 Bit
//                      123456789 123456789
#define IR_1          0b00111001110100000000,20
#define IR_2          0b00111001110100000001,20
#define IR_3          0b00111001110100000010,20
#define IR_4          0b00111001110100000011,20
#define IR_5          0b00111001110100000100,20
#define IR_6          0b00111001110100000101,20
#define IR_7          0b00111001110100000110,20
#define IR_8          0b00111001110100000111,20
#define IR_9          0b00111001110100001000,20
#define IR_0          0b00111001110100001001,20
#define IR_ENTER      0b00111001110101111100,20
#define IR_ARR_UP     0b00111001110101111000,20
#define IR_ARR_DOWN   0b00111001110101111001,20
#define IR_ARR_RIGHT  0b00111001110101110011,20
#define IR_ARR_LEFT   0b00111001110101110010,20
#define IR_CD_PLAY    0b00111001110100110010,20
#define IR_CD_PAUSE   0b00111001110100111001,20
#define IR_CD_STOP    0b00111001110100111000,20
#define IR_CD_PREV    0b00111001110100110000,20
#define IR_CD_NEXT    0b00111001110100110001,20
#define IR_CD_REW     0b00111001110100110011,20
#define IR_CD_FF      0b00111001110100110100,20 

#define IR_PLUS       0b00111001110100101110,20
#define IR_MINUS      0b00111001110100101111,20

    
#define   DO_IR_ON    PORTB|= BIT0
#define   DO_IR_OFF   PORTB&=~BIT0

//--------------------------------------------------------
void sendIr(U32 value, U8 bitcount){
//--------------------------------------------------------
register U32 mask=1;
register U8 puls;

  DDRB = BIT0; //Ausgang einschalten
//Preamble: 2400s = 96 Pulses
  for (puls=0; puls<96; puls++){
    DO_IR_ON;
    _delay_us(10);
    DO_IR_OFF;
    _delay_us(14);
  }
  _delay_us(599);

  while (bitcount--){
    if (value & mask){
      for (puls=0; puls<48; puls++){ //1200s = 48 pulses
        DO_IR_ON;
        _delay_us(10);
        DO_IR_OFF;
        _delay_us(14);
      }
    }else{
      for (puls=0; puls<24; puls++){//600s = 24 pulses
        DO_IR_ON;
        _delay_us(10);
        DO_IR_OFF;
        _delay_us(14);
      }
    }
    mask<<=1;
    _delay_us(599);
  }
  DDRB = 0; //Ausgang abschalten
}
//--------------------------------------------------------
void sendIr3(U32 value, U8 bitcount){
//--------------------------------------------------------
  sendIr(value, bitcount);
  _delay_ms(25);
  sendIr(value, bitcount);
  _delay_ms(25);
  sendIr(value, bitcount);
  _delay_ms(25);
}

ISR(PCINT0_vect){
//--------------------------------------------------------
  //Mache nix. Interrupt dient nur zum aufwachen.
}
ISR(PCINT1_vect){
//--------------------------------------------------------
  //Mache nix. Interrupt dient nur zum aufwachen.
}

//--------------------------------------------------------
int main(void){
//--------------------------------------------------------
register U16 key;
register U16 lastKey=0;
register U8 sleepAllowed=1;
U8 a;  

  DDRA = 0;
  PORTA = 0xFF;  //Pullup an alle Eingnge
  DDRB = BIT0; //LED Ausgang 
  PORTB= BIT1|BIT2|BIT3;  //Pullup an Eingnge
  DO_IR_OFF;
  PRR=(1<<PRTIM1)|(1<<PRTIM0)|(1<<PRUSI)|(1<<PRADC); //Strom sparen: 4,9 statt 6,5mA

  //Pin change Interrupts wecken aus Power-Down Modus 
  GIMSK = (1<<PCIE0)|(1<<PCIE1); //all pin change interrupts enable
  PCMSK0 = 0xff; //sensitiv auf alle Eingnge an Port A
  PCMSK1 =(1<<PCINT9)|(1<<PCINT10); //sensitiv auf Tasten-Pins an PortB

  _delay_ms(80);
  sendIr(IR_VOL_DN);

  sei();

  while(1){  
    if (sleepAllowed){
      DO_IR_OFF; //Paranoia only
      DDRB = 0;
      MCUCR = (1<<BODS)|(1<<SE)|(1<<SM1); //Sleep enable; Power-Down Mode
      asm("SLEEP"); //Jetzt braucht der Chip nur noch ca. 0.0001mA (ca. 1 mAh pro Jahr)
        //Bei nderungen an PB2-3 oder PA0-7 wacht der Prozessor wieder auf.
    } 

    a = ~PINA;
    key = a;
    if (!(PINB & BIT1)) key+=256;
    if (!(PINB & BIT2)) key+=512;
    sleepAllowed=0;
    if (key!=lastKey){
      
      lastKey = key;
      switch(key){
        case 0: //Taste wurde losgelassen. Prellen abwarten.
          _delay_ms(100);
          break;
        case (BIT0): //prev
          sendIr3(IR_CD_PREV);
          _delay_ms(100);
          break;
        case (BIT1): //next
          sendIr3(IR_CD_NEXT);
          _delay_ms(100);
          break;
        case (BIT0|BIT1): //Pause (Tasten next+prev) 
          sendIr3(IR_CD_PAUSE);
          _delay_ms(100);
          break;
        case (BIT2): //Play CD
          sendIr3(IR_CD);
          _delay_ms(100);
          sendIr3(IR_CD_PLAY);
          break;
        case (BIT3): //USB
          sendIr3(IR_USB);
          _delay_ms(100);
          sendIr3(IR_CD_PLAY);
          break;
        case (BIT2|BIT3): //Sleep  (Tasten CD+USB)
          for (a=0; a<9; a++){
            sendIr3(IR_SLEEP); //Beim ersten mal: Auf Sleep 20Minuten (dann 50, 80, off)
            _delay_ms(100);
          }
          break;
          
        case (BIT4): //Tuner Prg.5
          sendIr3(IR_TUNER);
          _delay_ms(200);
          sendIr3(IR_5);
          _delay_ms(200);
          sendIr3(IR_ENTER);
          break;
        case (BIT5): //Tuner Prg.2
          sendIr3(IR_TUNER);
          _delay_ms(200);
          sendIr3(IR_2);
          _delay_ms(200);
          sendIr3(IR_ENTER);
          break;
        case (BIT6): //Tuner Prg.1
          sendIr3(IR_TUNER);
          _delay_ms(200);
          sendIr3(IR_1);
          _delay_ms(200);
          sendIr3(IR_ENTER);
          break;
        case (BIT7): //Volume+
          sendIr(IR_VOL_UP);
          lastKey=9999; //Damit VolUp periodisch gesendet wird 
          _delay_ms(25);
          break;
        case (BIT9): //Volume-
          sendIr(IR_VOL_DN);
          lastKey=9999; //Damit VolDown periodisch gesendet wird 
          _delay_ms(25);
          break;
        case (BIT8): //Power
          sendIr3(IR_POWER);
          break;
      }//switch

    }else{
      sleepAllowed=1;
    }
  }//While 1
  
  return 0;
}
// EOF----------------------------------------------------------------------------------------------------------
